/**
 ****************************************************************************
 * @file    tmpm3u6_adc.h
 * @brief   This file provides all the functions prototypes for ADC driver.
 * @version V2.0.2.5
 * @date    2014/04/23
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __TMPM3U6_ADC_H
#define __TMPM3U6_ADC_H

#ifdef __cplusplus
extern "C" {
#endif

/* Includes ------------------------------------------------------------------*/
#include "TMPM3U6.h"
#include "tx03_common.h"

#if defined(__TMPM3U6_ADC_H)
/** @addtogroup TX03_Periph_Driver
  * @{
  */

/** @addtogroup ADC
  * @{
  */

/** @addtogroup ADC_Exported_types
  * @{
  */


/* macro to set Analog Input channel or Program number together with its Enable/Disable setting for the trigger registers */
#define  TRG_ENABLE(AINx_ProgX)  ((uint8_t)( 0x80U | (AINx_ProgX)))
#define  TRG_DISABLE(AINx_ProgX) ((uint8_t)(AINx_ProgX))


#define ADC_HOLD_FIX                ((uint32_t)0x09U << 3U)     /* must write 0b1001 to this area */
#define IS_ADC_SAMPLEHOLD(param)    ((param) == ADC_HOLD_FIX)

#define ADC_FC_DIVIDE_LEVEL_NONE    ((uint32_t)0x00U)
#define IS_ADC_PRESCALER(param)     ((param) == ADC_FC_DIVIDE_LEVEL_NONE)



    typedef enum {
        ADC_TRG_SW = 0U,
        ADC_TRG_CONSTANT = 1U,
        ADC_TRG_TIMER = 2U,
        ADC_TRG_PMD = 3U
    } ADC_TrgType;
#define IS_ADC_TRGTYPE(param)       ((param) <= ADC_TRG_PMD)
#define IS_ADC_START(param)         ((param) <= ADC_TRG_CONSTANT)

    typedef enum {
        ADC_10BITS = 0U,
        ADC_12BITS = 1U,
        ADC_RESERVEDBITS = 2U
    } ADC_Resolution;
#define IS_ADC_RESOLUTION(param)  ((param) <= ADC_12BITS)


    typedef enum {
        ADC_CMPCR_0 = 0U,
        ADC_CMPCR_1 = 1U
    } ADC_CMPCRx;
#define IS_ADC_CMPCRx(param)    ((param) <= ADC_CMPCR_1)

    typedef enum {
        ADC_REG0 = 0U,
        ADC_REG1 = 1U,
        ADC_REG2 = 2U,
        ADC_REG3 = 3U,
        ADC_REG4 = 4U,
        ADC_REG5 = 5U,
        ADC_REG6 = 6U,
        ADC_REG7 = 7U,
        ADC_REG8 = 8U,
        ADC_REG9 = 9U,
        ADC_REG10 = 10U,
        ADC_REG11 = 11U
    } ADC_REGx;
#define IS_ADC_REGx(param)      ((param) <= ADC_REG11)


    typedef enum {
        ADC_LARGER_THAN_CMP_REG = 0U,
        ADC_SMALLER_THAN_CMP_REG = 1U
    } ADC_CmpCondition;
#define IS_ADC_CMPCONDITION(param)   ((param) <= ADC_SMALLER_THAN_CMP_REG)


/** 
  * @brief  ADC Monitor Init Structure definition
  */
    typedef struct {
        ADC_CMPCRx CMPCRx;      /*!< Select ADC_CMPCR_0 for ADCMPCR0 or ADC_CMPCR_1 for ADCMPCR1 */
        ADC_REGx ResultREGx;    /*!< Select which result register among ADC_REG0 to ADC_REG11 will be used */
        uint32_t CmpTimes;      /*!< How many times will Comparison be counted, range from 1 to 16 */
        ADC_CmpCondition Condition;     /*!< Conditon to compare ADREGx with ADCMPy. ( x = 0 to 11, y = 0 to 1 ) */
        uint32_t CmpValue;      /*!< Comparison value to be set in ADCMP0 or ADCMP1, value max for 12bit mode is 4095, for 10bit mode is 1023 */
    } ADC_MonitorTypeDef;

#define IS_ADC_CMPTIMES(param)  (((param) >= 1U) && ((param) <= 16U))
#define IS_ADC_CMPVALUE_12BIT(param)   ((param) <= 4095U)
#define IS_ADC_CMPVALUE_10BIT(param)   ((param) <= 1023U)


    typedef enum {
        PMD_TRG_PROG_SEL0 = 0U,
        PMD_TRG_PROG_SEL1 = 1U,
        PMD_TRG_PROG_SEL2 = 2U,
        PMD_TRG_PROG_SEL3 = 3U
    } PMD_TRG_PROG_SELx;
#define IS_PMD_TRG_PROG_SELx(param)   ((param) <= PMD_TRG_PROG_SEL3)

    typedef enum {
        PMD_PROG0 = 0U,
        PMD_PROG1 = 1U,
        PMD_PROG2 = 2U,
        PMD_PROG3 = 3U,
        PMD_PROG4 = 4U,
        PMD_PROG5 = 5U
    } PMD_PROGx;
#define IS_PMD_PROGx(param)   ((param) <= PMD_PROG5)

/* ProgX to be set for ADPSELx, use with macro TRG_ENABLE() and TRG_DISABLE() */
#define IS_PMD_PROGx_ENABLE_DISABLE(param)\
                                 ( (((param) >= TRG_ENABLE(PMD_PROG0))  && \
                                    ((param) <= TRG_ENABLE(PMD_PROG5)) ) || \
                                    ((param) <= TRG_DISABLE(PMD_PROG5)) )

    typedef enum {
        PMD_INTNONE = 0U,
        PMD_INTADPD0 = 1U,
        PMD_INTADPD1 = 2U
    } PMD_INT_NAME;
#define IS_PMD_INT_NAME(param)   ((param) <= PMD_INTADPD1)

    /*!< to select and set the interrupt to be generated for program 0 to 5 */
    typedef struct {
        PMD_INT_NAME INTProg0;
        PMD_INT_NAME INTProg1;
        PMD_INT_NAME INTProg2;
        PMD_INT_NAME INTProg3;
        PMD_INT_NAME INTProg4;
        PMD_INT_NAME INTProg5;
    } PMD_TrgProgINTTypeDef;

    typedef enum {
        ADC_AIN0 = 0U,          /*!< define for Analog Input channel */
        ADC_AIN1 = 1U,
        ADC_AIN2 = 2U,
        ADC_AIN3 = 3U,
        ADC_AIN4 = 4U,
        ADC_AIN5 = 5U,
        ADC_AIN6 = 6U,
        ADC_AIN7 = 7U,
        ADC_AIN8 = 8U,
        ADC_AIN9 = 9U,
        ADC_AIN10 = 10U,
        ADC_AIN11 = 11U,
        ADC_AIN12 = 12U,
        ADC_AIN13 = 13U,
        ADC_AIN14 = 14U,
        ADC_AIN15 = 15U,
        ADC_AIN16 = 16U,
        ADC_AIN17 = 17U
    } ADC_AINx;

/* Analog Input channel to be set for trigger register, use with macro TRG_ENABLE() and TRG_DISABLE() */
#define IS_ADC_AINx(param)   (( ((param) >= TRG_ENABLE(ADC_AIN0))  && \
                                ((param) <= TRG_ENABLE(ADC_AIN17)) ) || \
                                ((param) <= TRG_DISABLE(ADC_AIN17)) )


/**
  * @brief ADC PMD Trigger Program Register Setting Structure definition
  *   Reg0_AINx to Reg3_AINx must be inputed with macro as the format below:
  *     TRG_ENABLE(y), TRG_DISABLE(y).
  *   'y' above can be one of the following values:
  *     ADC_AIN0 to ADC_AIN17
  */
    typedef struct {
        PMD_PROGx ProgNum;      /*!< select PROG0 to PROG5 for ADPSETx , x = 0 to 5 */
        uint8_t Reg0_AINx;      /*!< select AI channel together with its enabled or disabled setting for REG0 in ADPSETx */
        uint8_t Reg1_AINx;      /*!< select AI channel together with its enabled or disabled setting for REG1 in ADPSETx */
        uint8_t Reg2_AINx;      /*!< select AI channel together with its enabled or disabled setting for REG2 in ADPSETx */
        uint8_t Reg3_AINx;      /*!< select AI channel together with its enabled or disabled setting for REG3 in ADPSETx */
    } PMD_TrgTypeDef;


/**
  * @brief Union to store ADC result
  */
    typedef union {
        uint32_t All;
        struct {
            uint32_t Stored:1;  /*!< bit0, AD result has been stored */
            uint32_t OverRun:1; /*!< bit1, OverRun, new AD result is stored before the old one been read */
            uint32_t Reserved1:2;       /*!< bit2 to bit3, reserved */
            uint32_t ADResult:12;       /*!< bit4 to bit15, store AD result */
            uint32_t Reserved2:16;      /*!< bit16 to bit31, reserved */
        } Bit;
    } ADC_Result;

/** @} */
/* End of group ADC_Exported_types */

/** @defgroup ADC_Exported_FunctionPrototypes
  * @{
  */
    void ADC_SetClk(uint32_t Sample_HoldTime, uint32_t Prescaler_Output);
    void ADC_SetResolution(ADC_Resolution ADBits);
    ADC_Resolution ADC_GetResolution(void);
    void ADC_Enable(void);
    void ADC_Disable(void);
    void ADC_Start(ADC_TrgType Trg);
    void ADC_StopConstantTrg(void);
    WorkState ADC_GetConvertState(ADC_TrgType Trg);
    void ADC_SetLowPowerMode(FunctionalState NewState);
    void ADC_SetMonitor(ADC_MonitorTypeDef * Monitor);
    void ADC_DisableMonitor(ADC_CMPCRx CMPCRx);
    ADC_Result ADC_GetConvertResult(ADC_REGx ResultREGx);
    void ADC_SelectPMDTrgProgNum(PMD_TRG_PROG_SELx SELx, uint8_t MacroProgNum);
    void ADC_SetPMDTrgProgINT(PMD_TrgProgINTTypeDef * TrgProgINT);
    void ADC_SetPMDTrg(PMD_TrgTypeDef * PMDTrg);
    void ADC_SetTimerTrg(ADC_REGx ResultREGx, uint8_t MacroAINx);
    void ADC_SetSWTrg(ADC_REGx ResultREGx, uint8_t MacroAINx);
    void ADC_SetConstantTrg(ADC_REGx ResultREGx, uint8_t MacroAINx);


/** @} */
/* End of group ADC_Exported_FunctionPrototypes */

/** @} */
/* End of group ADC */

/** @} */
/* End of group TX03_Periph_Driver */

#endif                          /* __TMPM3U6_ADC_H */

#ifdef __cplusplus
}
#endif
#endif /*__TMPM3U6_ADC_H */
