/**
 *******************************************************************************
 * @file    main.c
 * @brief   the application functions of DMAC UART demo for the
 *          TOSHIBA 'TMPM3U6' Device Series
 * @version V0.101
 * @date    2019/07/09
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */

#include <stdio.h>
#include <string.h>
#include "main.h"

#define UART0_BASE_ADDR   TSB_SC0_BASE
#define UART0_BUF_OFFSET  sizeof(UART0->EN)
typedef enum { FAILED = 0, PASSED = !FAILED } TestResult;

/* Function declaration */

/* User ram definition */
const uint8_t SRC_Buffer[] = "TOSHIBA";

volatile uint8_t UART_Rx_Buffer[BUFFER_SIZE];
volatile uint8_t UART_Rx_Cnt;
volatile TestResult TransferResult = FAILED;

/* main function */
int main(void)
{
    UART_InitTypeDef UART_InitStruct;
    DMAC_InitTypeDef DMAC_InitStruct;
    DMAC_Channel myChannel = DMAC_CHANNEL_1;
    uint32_t LinkAddr;
    uint8_t size;

    /* Set GPIO as UART */
    GPIO_SetUART();

    /* Set UART1 as reception */
    UART_Rx_Cnt = 0U;
    UART_InitStruct.BaudRate = 115200U;
    UART_InitStruct.DataBits = UART_DATA_BITS_8;
    UART_InitStruct.StopBits = UART_STOP_BITS_1;
    UART_InitStruct.Parity = UART_NO_PARITY;
    UART_InitStruct.Mode = UART_ENABLE_RX;
    UART_InitStruct.FlowCtrl = UART_NONE_FLOW_CTRL;
    UART_Enable(UART1);
    UART_Init(UART1, &UART_InitStruct);

    /* Set UART0 as transmitter */
    UART_InitStruct.Mode = UART_ENABLE_TX;
    UART_Enable(UART0);
    UART_Init(UART0, &UART_InitStruct);

    /* DMA channel1 configuration */
    size = sizeof(SRC_Buffer);
    DMAC_InitStruct.SrcAddr = (uint32_t) SRC_Buffer;
    DMAC_InitStruct.SrcIncrementState = ENABLE;
    DMAC_InitStruct.SrcBitWidth = DMAC_BYTE;
    DMAC_InitStruct.SrcBurstSize = DMAC_1_BEAT;
    DMAC_InitStruct.DstAddr = (uint32_t) (UART0_BASE_ADDR + UART0_BUF_OFFSET);
    DMAC_InitStruct.DstIncrementState = DISABLE;
    DMAC_InitStruct.DstBitWidth = DMAC_BYTE;
    DMAC_InitStruct.DstBurstSize = DMAC_1_BEAT;
    DMAC_InitStruct.TxSize = size;
    DMAC_InitStruct.TxDirection = DMAC_MEMORY_TO_PERIPH;
    DMAC_InitStruct.TxPeriph = DMAC_SIO_0_RTX;

    /* CXCONTROL<I> must be set when use transfer interrupt */
    DMAC_InitStruct.TxINT = ENABLE;

    LinkAddr = 0U;

    __disable_irq();
    /* enable DMA interrupt */
    NVIC_ClearPendingIRQ(INTDMACTC_IRQn);
    NVIC_EnableIRQ(INTDMACTC_IRQn);

    /*Enable DMA circuit */
    DMAC_Enable();
    /* Clear any pending interrupts on the channel */
    DMAC_ClearTxINTReq(myChannel, DMAC_INT_TX_END);
    DMAC_ClearTxINTReq(myChannel, DMAC_INT_TX_ERR);
    /*Initialize DMA channel */
    DMAC_Init(myChannel, &DMAC_InitStruct);
    /* If don't require scatter/gather function, set the DMACCxLLI Register to 0 */
    DMAC_SetLinkedList(myChannel, LinkAddr);
    __enable_irq();

    /* enable UART1 reception interrupt */
    NVIC_ClearPendingIRQ(INTRX1_IRQn);
    NVIC_EnableIRQ(INTRX1_IRQn);

    /*Enable DMA Channel transfer end interrupt */
    DMAC_SetTxINTConfig(myChannel, DMAC_INT_TX_END, ENABLE);
    /* Enable DMA burst transfer request */
    DMAC_SetSWBurstReq(DMAC_SIO_0_RTX);

    /*Enable DMA channel */
    DMAC_SetDMAChannel(myChannel, ENABLE);

    /* Wait the end of transmission */
    while (TxEndFlag != DONE) {
        /* Do nothing */
    }
    while (UART_Rx_Cnt < size) {
        /* Do nothing */
    }

    while (1) {
        /* Do nothing */
    }
}

/**
  * @brief  Configure I/O ports for UART
  * @param  None
  * @retval None
  */
void GPIO_SetUART(void)
{
    /* Set PortE 0 as TXD0 */
    GPIO_SetOutputEnableReg(GPIO_PE, GPIO_BIT_0, ENABLE);
    GPIO_EnableFuncReg(GPIO_PE, GPIO_FUNC_REG_1, GPIO_BIT_0);

    /* Set PortE 1 as RXD0 */
    GPIO_SetInputEnableReg(GPIO_PE, GPIO_BIT_1, ENABLE);
    GPIO_EnableFuncReg(GPIO_PE, GPIO_FUNC_REG_1, GPIO_BIT_1);

    /* Set PortA 5 as TXD1 */
    GPIO_SetOutputEnableReg(GPIO_PA, GPIO_BIT_5, ENABLE);
    GPIO_EnableFuncReg(GPIO_PA, GPIO_FUNC_REG_1, GPIO_BIT_5);

    /* Set PortA 6 as RXD1 */
    GPIO_SetInputEnableReg(GPIO_PA, GPIO_BIT_6, ENABLE);
    GPIO_EnableFuncReg(GPIO_PA, GPIO_FUNC_REG_1, GPIO_BIT_6);
}

#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
