/**
 *******************************************************************************
 * @file    main.c
 * @brief   the application functions of PMD_PHASE_OUTPUT demo for the TOSHIBA 'TMPM3U6'
 *          Device Series
 * @version V0.101
 * @date    2019/07/09
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */

/* Include -------------------------------------------------------------------*/
#include "main.h"

/* Global variable define ----------------------------------------------------*/
volatile uint32_t gState = 0U;

/* Choose one demo to define */
#define DEMO_U_PHASE         /* U-, V-, W- phases are same in this demo */
//#define DEMO_3_PHASE         /* U-, V-, W- phases are not same in this demo */

/**
  * @brief  Main function
  * @param  None
  * @retval None
  */
int main(void)
{
    /* LED initialization */
    LEDInit();

    /* GPIO configuration */
    PMD_IO_Config();

    /* PMD PWM configuration */
    PMD_PWM_Config();

    /* Enable PMD1 */
    PMD_Enable(PMD1);

    while(1){
        /* Get the EMG protection condition*/
        gState = PMD_GetEMGCondition(PMD1);
        /* Judge the EMG protection condition */
        if (gState == EMG_PROTECTION) {
            /* LED3 will light on if the condition is in protection */
            LedOn(LED3);
            /* Back EMG protection to normal */
            PMD_BackToNormal();
            Delay(1000U);
        } else {
            /* LED3 will light off if the condition is not in protection */
            LedOff(LED3);
        }
    }
}

/**
  * @brief  PMD PWM configuration
  * @param  None.
  * @retval None.
  */
void PMD_PWM_Config(void)
{
    PMD_InitTypeDef m_pmd;

    /* PMD1 initialization */
    m_pmd.CycleMode = PMD_PWM_NORMAL_CYCLE;
#ifdef DEMO_U_PHASE
    m_pmd.DutyMode = PMD_DUTY_MODE_U_PHASE;         /* U-phase in common */
#endif
#ifdef DEMO_3_PHASE
    m_pmd.DutyMode = PMD_DUTY_MODE_3_PHASE;         /* 3-phase independent */
#endif
    m_pmd.IntTiming = PMD_PWM_INT_TIMING_MINIMUM;
    m_pmd.IntCycle = PMD_PWM_INT_CYCLE_1;
    m_pmd.CarrierMode = PMD_CARRIER_WAVE_MODE_1;    /* PWM mode 1 (center PWM, triangle wave) */
    m_pmd.CycleTiming = 0x3FFFU;

    PMD_Init(PMD1,&m_pmd);

    PMD_SetAllPhaseCompareValue(PMD1,0x0FFFU,0x1FFFU,0x2FFFU);
    PMD_SetPortOutput(PMD1,PMD_PHASE_ALL,PMD_OUTPUT_PWM_IPWM);
    PMD_SetPortControl(PMD1,PMD_PORT_PMD_OUTPUT);
    PMD_SetEMGMode(PMD1, PMD_EMG_MODE_0);
    PMD_EnableEMG(PMD1);
    PMD_SetDeadTime(PMD1,DEAD_TIME);
    PMD_SetEMGNoiseElimination(PMD1,PMD_NOISE_ELIMINATION_240);
}

/**
  * @brief  GPIO configuration
  * @param  None.
  * @retval None.
  */
void PMD_IO_Config(void)
{
    /* Set PG0-PG5 as output */
    GPIO_SetOutputEnableReg(GPIO_PG, GPIO_BIT_0 | GPIO_BIT_1 | GPIO_BIT_2 | GPIO_BIT_3 | GPIO_BIT_4 | GPIO_BIT_5, ENABLE);
    /* Set PG0-PG5 as UO1 X01 VO1 YO1 WO1 ZO1 */
    GPIO_EnableFuncReg(GPIO_PG, GPIO_FUNC_REG_1, GPIO_BIT_0 | GPIO_BIT_1 | GPIO_BIT_2 | GPIO_BIT_3 | GPIO_BIT_4 | GPIO_BIT_5);
    /* Set PG6 as input */
    GPIO_SetInputEnableReg(GPIO_PG, GPIO_BIT_6, ENABLE);
    /* Set PG6 as EMG1 */
    GPIO_EnableFuncReg(GPIO_PG, GPIO_FUNC_REG_1, GPIO_BIT_6);

    /* Before config PG6 as EMG1 input, thg EMG1 input level is "L" and the EMG1 is in protection. */
    /* Therefore, it is necessary to release EMG protection status. */
    PMD_EMGRelease(PMD1);
}

/**
  * @brief  Back EMG protection to normal
  * @param  None.
  * @retval None.
  */
void PMD_BackToNormal(void)
{
    PMD_EMGRelease(PMD1);
    /* After calling the function PMD_EMGRelease(), the output selection is cleared to 0 */
    /* Therefore, it is necessary to set again */
    PMD_SetPortOutput(PMD1,PMD_PHASE_ALL,PMD_OUTPUT_PWM_IPWM);
}

/**
  * @brief  Delay time
  * @param  time.
  * @retval None.
  */
void Delay(uint32_t time)
{
    volatile uint32_t TimeCounter = 0U;
    uint32_t DelayTime = 0U;

    DelayTime = time * 800U;
    for (TimeCounter = 0U; TimeCounter < DelayTime; TimeCounter++) {
        /* Do nothing */
    }
}

#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    __NOP();
}
#endif
