/**
 *******************************************************************************
 * @file    tmpm3u6_dmac_int.c
 * @brief   test M3U6 SPI Host to Slave (SSP0 to SSP1)
 *           for the TOSHIBA 'TMPM3U6' Device Series 
 * @version V0.111
 * @date    2019/07/09
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "tmpm3u6_ssp.h"
#include "tmpm3u6_gpio.h"
#include "dma.h"

typedef enum { SAME = 1U, NOT_SAME = !SAME } TestResult;

void DisplayLED(uint16_t dat);
void initSSP(void);
void GPIO_SetSSP(void);
TestResult Buffercompare(const uint16_t * Src, const uint16_t * Dst, uint32_t Len);

#define MAX_BUFSIZE  64U

TestResult result = NOT_SAME;

/* to check the effect due to too low speed, uncomment the line below */
/* #define BITRATE_MIN */

/*------------------------------------------------------------------------
	SSP0 pins	            Note	                SSP1 pins
1	PC3/SP0FSS  (CN9 43)	Connect PC3 to PN3	    PN3/SP1FSS  (CN4 21)	
2	PC2/SP0CLK  (CN9 41)	Connect PC2 to PN2	    PN2/SP1CLK  (CN4 22)	
3	PC1/SP0DI   (CN9 37)	Connect PC1 to PN0	    PN1/SP1DI   (CN4 20)	
4	PC0/SP0DO   (CN9 33)	Connect PC0 to PN1	    PN0/SP1DO   (CN4 18)	
	Connect CN9 43 to CN4 21 , CN9 41 to CN4 22 , CN9 37 to CN4 18.
    Connect CN9 33 to CN4 20
 
 
CR.x: call GPIO_SetOutputEnableReg()
PC0F2: call: GPIO_EnableFuncReg()
IE.x: call: GPIO_SetInputEnableReg()
OD.x: call: GPIO_SetOpenDrain()
 
to use GPIO as SSP0 (SPI Host), 
//========
PC0 to be used as SP0DO:
 set: PCCR.0,  GPIO_SetOutputEnableReg(GPIO_PC, GPIO_BIT_0, ENABLE);
 set: PC0F2,   GPIO_EnableFuncReg(GPIO_PC, GPIO_FUNC_REG_2, GPIO_BIT_0);
 clear: PCIE.0, GPIO_SetInputEnableReg(GPIO_PC, GPIO_BIT_0, DISABLE);

PC1 to be used as SP0DI: 
 set: PCIE.1,   GPIO_SetInputEnableReg(GPIO_PC, GPIO_BIT_1, ENABLE);
 set: PC1F2     GPIO_EnableFuncReg(GPIO_PC, GPIO_FUNC_REG_2, GPIO_BIT_1);
 clear: PCCR.1, GPIO_SetOutputEnableReg(GPIO_PC, GPIO_BIT_1, DISABLE);
 clear: PCOD.1  GPIO_SetOpenDrain(GPIO_PC, GPIO_BIT_1, DISABLE);

PC2 to be used as SP0CLK out: 
 set: PCCR.2,   GPIO_SetOutputEnableReg(GPIO_PC, GPIO_BIT_2, ENABLE);
 set: PC2F2     GPIO_EnableFuncReg(GPIO_PC, GPIO_FUNC_REG_2, GPIO_BIT_2);
 clear: PCIE.2  GPIO_SetInputEnableReg(GPIO_PC, GPIO_BIT_2, DISABLE);

PC3 to be used as SP0FSS out:
 set: PCCR.3,   GPIO_SetOutputEnableReg(GPIO_PC, GPIO_BIT_3, ENABLE);
 set: PC3F2     GPIO_EnableFuncReg(GPIO_PC, GPIO_FUNC_REG_2, GPIO_BIT_3);
 clear: PCIE.3  GPIO_SetInputEnableReg(GPIO_PC, GPIO_BIT_3, DISABLE); 
//========

to use GPIO as SSP1 (SPI Slave), 
//========================= 
PN0 to be used as SP1DO:
 set: PNCR.0,   GPIO_SetOutputEnableReg(GPIO_PN, GPIO_BIT_0, ENABLE);
 set: PN0F1,    GPIO_EnableFuncReg(GPIO_PN, GPIO_FUNC_REG_1, GPIO_BIT_0);
 clear: PNIE.0  GPIO_SetInputEnableReg(GPIO_PN, GPIO_BIT_0, DISABLE);

PN1 to be used as SP1DI: 
 set: PNIE.1,   GPIO_SetInputEnableReg(GPIO_PN, GPIO_BIT_1, ENABLE);
 set: PN1F1     GPIO_EnableFuncReg(GPIO_PN, GPIO_FUNC_REG_1, GPIO_BIT_1);
 clear: PNCR.1, GPIO_SetOutputEnableReg(GPIO_PN, GPIO_BIT_1, DISABLE);
 clear: PNOD.1  GPIO_SetOpenDrain(GPIO_PN, GPIO_BIT_1, DISABLE);

PN2 to be used as SP1CLK in: 
 set: PCIE.2,   GPIO_SetInputEnableReg(GPIO_PN, GPIO_BIT_2, ENABLE);
 set: PN2F1     GPIO_EnableFuncReg(GPIO_PN, GPIO_FUNC_REG_1, GPIO_BIT_2);
 clear: PNCR.2, GPIO_SetOutputEnableReg(GPIO_PN, GPIO_BIT_2, DISABLE);
 clear: PNOD.2, GPIO_SetOpenDrain(GPIO_PN, GPIO_BIT_2, DISABLE);

PN3 to be used as SP1FSS in:
 set: PCIE.3,   GPIO_SetInputEnableReg(GPIO_PN, GPIO_BIT_3, ENABLE);        
 set: PN3F1     GPIO_EnableFuncReg(GPIO_PN, GPIO_FUNC_REG_1, GPIO_BIT_3);  
 clear: PNCR.3, GPIO_SetOutputEnableReg(GPIO_PN, GPIO_BIT_3, DISABLE);     
 clear: PNOD.3, GPIO_SetOpenDrain(GPIO_PN, GPIO_BIT_3, DISABLE);       
    
------------------------------------------------------------------------*/
void GPIO_SetSSP(void)
{
    /* PC0 to be used as SP0DO: */
    GPIO_SetOutputEnableReg(GPIO_PC, GPIO_BIT_0, ENABLE);
    GPIO_EnableFuncReg(GPIO_PC, GPIO_FUNC_REG_2, GPIO_BIT_0);
    GPIO_SetInputEnableReg(GPIO_PC, GPIO_BIT_0, DISABLE);

    /* PC1 to be used as SP0DI: */
    GPIO_SetInputEnableReg(GPIO_PC, GPIO_BIT_1, ENABLE);
    GPIO_EnableFuncReg(GPIO_PC, GPIO_FUNC_REG_2, GPIO_BIT_1);
    GPIO_SetOutputEnableReg(GPIO_PC, GPIO_BIT_1, DISABLE);
    GPIO_SetOpenDrain(GPIO_PC, GPIO_BIT_1, DISABLE);

    /* PC2 to be used as SP0CLK out: */
    GPIO_SetOutputEnableReg(GPIO_PC, GPIO_BIT_2, ENABLE);
    GPIO_EnableFuncReg(GPIO_PC, GPIO_FUNC_REG_2, GPIO_BIT_2);
    GPIO_SetInputEnableReg(GPIO_PC, GPIO_BIT_2, DISABLE);

    /* PC3 to be used as SP0FSS out: */
    GPIO_SetOutputEnableReg(GPIO_PC, GPIO_BIT_3, ENABLE);
    GPIO_EnableFuncReg(GPIO_PC, GPIO_FUNC_REG_2, GPIO_BIT_3);
    GPIO_SetInputEnableReg(GPIO_PC, GPIO_BIT_3, DISABLE); 



    /* PN0 to be used as SP1DO: */
    GPIO_SetOutputEnableReg(GPIO_PN, GPIO_BIT_0, ENABLE);
    GPIO_EnableFuncReg(GPIO_PN, GPIO_FUNC_REG_1, GPIO_BIT_0);
    GPIO_SetInputEnableReg(GPIO_PN, GPIO_BIT_0, DISABLE);

    /* PN1 to be used as SP1DI: */
    GPIO_SetInputEnableReg(GPIO_PN, GPIO_BIT_1, ENABLE);
    GPIO_EnableFuncReg(GPIO_PN, GPIO_FUNC_REG_1, GPIO_BIT_1);
    GPIO_SetOutputEnableReg(GPIO_PN, GPIO_BIT_1, DISABLE);
    GPIO_SetOpenDrain(GPIO_PN, GPIO_BIT_1, DISABLE);

    /* PN2 to be used as SP1CLK in: */
    GPIO_SetInputEnableReg(GPIO_PN, GPIO_BIT_2, ENABLE);
    GPIO_EnableFuncReg(GPIO_PN, GPIO_FUNC_REG_1, GPIO_BIT_2);
    GPIO_SetOutputEnableReg(GPIO_PN, GPIO_BIT_2, DISABLE);
    GPIO_SetOpenDrain(GPIO_PN, GPIO_BIT_2, DISABLE);

    /* PN3 to be used as SP1FSS in: */
    GPIO_SetInputEnableReg(GPIO_PN, GPIO_BIT_3, ENABLE);        
    GPIO_EnableFuncReg(GPIO_PN, GPIO_FUNC_REG_1, GPIO_BIT_3);  
    GPIO_SetOutputEnableReg(GPIO_PN, GPIO_BIT_3, DISABLE);     
    GPIO_SetOpenDrain(GPIO_PN, GPIO_BIT_3, DISABLE);  
    
}    

void initSSP(void)
{
    SSP_InitTypeDef initSSP;
        
    /* init SSP0 as SPI Host and SSP1 as SPI Slave */
    
    /* configure the SSP module */
    initSSP.FrameFormat = SSP_FORMAT_SPI;

    /* set bit rate */
    initSSP.PreScale = 8U;
    initSSP.ClkRate = 8U;
    /* define BITRATE_MIN to run at minimum bit rate */
    /*   BitRate = fSYS / (PreScale x (1 + ClkRate)) */
#ifdef BITRATE_MIN
    initSSP.PreScale = 254U;
    initSSP.ClkRate = 255U;
#endif
    initSSP.ClkPolarity = SSP_POLARITY_LOW;
    initSSP.ClkPhase = SSP_PHASE_FIRST_EDGE;
    initSSP.DataSize = 16U;
    initSSP.Mode = SSP_MASTER;
    SSP_Init(TSB_SSP0, &initSSP);
    
    initSSP.ClkPhase = SSP_PHASE_FIRST_EDGE;
    initSSP.Mode = SSP_SLAVE;
    SSP_Init(TSB_SSP1, &initSSP);

    /* enable DMA in SSP module*/
    SSP_SetDMACtrl(TSB_SSP0, SSP_TX, ENABLE);
    SSP_SetDMACtrl(TSB_SSP1, SSP_RX, ENABLE);
    
    /* enable and run SSP module */
    SSP_Enable(TSB_SSP0);
    SSP_Enable(TSB_SSP1);
}

int main(void)
{
    GPIO_SetSSP();
    initSSP();
    InitDMA();
    
    /* Wait the end of transmission */
    while (TxEndFlag != DONE) {
        /* Do nothing */
    }
    
    /* now DMA is finished, Set a Break Point here, */
    /* after function Buffercompare() is called, result == SAME */
    result = Buffercompare( SRC_Buffer, DST_Buffer, BUFFER_SIZE);
    
    while(1) {
        /* do nothing */
    }
}

/**
  * @brief  To compare two half word buffer
  * @param  Src : pointer to point source buffer 
  * @param  Dst : pointer to point destination buffer   
  * @param  Len : how many data will be compared 
  * @retval SAME: two buffer contain same data
            NOT_SAME: two buffer contain different data  
  */
TestResult Buffercompare(const uint16_t * Src, const uint16_t * Dst, uint32_t Len)
{
    uint32_t tmp = 0U;


    for( tmp = 0U; tmp < Len ; tmp++ ) {
        if( Src[tmp] != Dst[tmp] ) {
             result = NOT_SAME;
             break;
       } else {
        result = SAME;
       }
    }

    return result;
}    

#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while(1){
        __NOP();
    }
}
#endif
